@@ -5,28 +5,40 @@ This module provides the low level functionality for running @initdb@, @postgres
 See 'startPlan' for more details.
 -}
 module Database.Postgres.Temp.Internal.Core where
-import qualified Database.PostgreSQL.Simple.Options as Client
-import qualified Database.PostgreSQL.Simple as PG
-import System.Process.Internals
-import System.Exit (ExitCode(..))
-import Data.String
-import System.Posix.Signals (sigINT, signalProcess)
-import Control.Exception
-import Data.Foldable (for_)
-import Control.Concurrent.Async (race_)
-import Control.Monad (forever, (>=>))
-import Control.Concurrent (threadDelay)
-import Data.Typeable
-import System.IO
-import System.Process
-import Data.Monoid
-import Text.PrettyPrint.ANSI.Leijen hiding ((<$>))
+
+import           Control.Concurrent (threadDelay)
+import           Control.Concurrent.Async (race_)
+import           Control.Exception
+import           Control.Monad (forever, (>=>))
 import qualified Data.ByteString.Char8 as BSC
+import           Data.Foldable (for_)
+import           Data.Monoid
+import           Data.String
+import           Data.Typeable
+import qualified Database.PostgreSQL.Simple as PG
+import qualified Database.PostgreSQL.Simple.Options as Client
+import           System.Exit (ExitCode(..))
+import           System.IO
+import           System.Posix.Signals (sigINT, signalProcess)
+import           System.Process
+import           System.Process.Internals
+import           Text.PrettyPrint.ANSI.Leijen hiding ((<$>))
 
 -- | Internal events for debugging
 data Event
-  = StartPostgres
+  = StartPlan String
+  -- ^ The first event. This useful for debugging
+  --   what is actual passed to the @initdb@, @createdb@ and
+  --   @postgres@.
+  | StartPostgres
+  -- ^ The second event. Postgres is about to get called
   | WaitForDB
+  -- ^ The third event. Postgres started. We are now about to
+  -- setup a reconnect loop (racing with a process checker)
+  | TryToConnect
+  -- ^ The fourth event and (possibly all subsequent events).
+  -- We are looping trying to successfully connect to the @postgres@
+  -- process.
   deriving (Show, Eq, Ord)
 
 -- | A list of failures that can occur when starting. This is not
@@ -42,8 +54,8 @@ data StartError
   | CreateDbFailed ExitCode
   -- ^ @createdb@ failed. This can be from invalid configuration or
   --   the database might already exist.
-  | CompletePlanFailed [String]
-  -- ^ The 'Database.Postgres.Temp.Partial.PartialPlan' was missing info and a complete 'Plan' could
+  | CompletePlanFailed String [String]
+  -- ^ The 'Database.Postgres.Temp.Config.Plan' was missing info and a complete 'CompletePlan' could
   --   not be created.
   deriving (Show, Eq, Ord, Typeable)
 
@@ -56,85 +68,90 @@ type Logger = Event -> IO ()
 -- | @postgres@ is not ready until we are able to successfully connect.
 --   'waitForDB' attempts to connect over and over again and returns
 --   after the first successful connection.
-waitForDB :: Client.Options -> IO ()
-waitForDB options = do
+waitForDB :: Logger -> Client.Options -> IO ()
+waitForDB logger options = do
+  logger TryToConnect
   let theConnectionString = Client.toConnectionString options
       startAction = PG.connectPostgreSQL theConnectionString
   try (bracket startAction PG.close mempty) >>= \case
-    Left (_ :: IOError) -> threadDelay 10000 >> waitForDB options
+    Left (_ :: IOError) -> threadDelay 10000 >> waitForDB logger options
     Right () -> return ()
 
--- | 'ProcessConfig' contains the configuration necessary for starting a
+-- | 'CompleteProcessConfig' contains the configuration necessary for starting a
 --   process. It is essentially a stripped down 'System.Process.CreateProcess'.
-data ProcessConfig = ProcessConfig
-  { processConfigEnvVars :: [(String, String)]
+data CompleteProcessConfig = CompleteProcessConfig
+  { completeProcessConfigEnvVars :: [(String, String)]
   -- ^ Environment variables
-  , processConfigCmdLine :: [String]
+  , completeProcessConfigCmdLine :: [String]
   -- ^ Command line arguements
-  , processConfigStdIn   :: Handle
+  , completeProcessConfigStdIn   :: Handle
   -- ^ The 'Handle' for standard input
-  , processConfigStdOut  :: Handle
+  , completeProcessConfigStdOut  :: Handle
   -- ^ The 'Handle' for standard output
-  , processConfigStdErr  :: Handle
+  , completeProcessConfigStdErr  :: Handle
   -- ^ The 'Handle' for standard error
   }
 
 prettyKeyPair ::(Pretty a, Pretty b) => a -> b -> Doc
 prettyKeyPair k v = pretty k <> text ": " <> pretty v
 
-instance Pretty ProcessConfig where
-  pretty ProcessConfig {..}
-    =  text "processConfigEnvVars:"
+instance Pretty CompleteProcessConfig where
+  pretty CompleteProcessConfig {..}
+    =  text "completeProcessConfigEnvVars:"
     <> softline
-    <> indent 2 (vsep (map (uncurry prettyKeyPair) processConfigEnvVars))
+    <> indent 2 (vsep (map (uncurry prettyKeyPair) completeProcessConfigEnvVars))
     <> hardline
-    <> text "processConfigCmdLine:"
+    <> text "completeProcessConfigCmdLine:"
     <> softline
-    <> text (unwords processConfigCmdLine)
+    <> text (unwords completeProcessConfigCmdLine)
 
 -- | Start a process interactively and return the 'ProcessHandle'
 startProcess
   :: String
   -- ^ Process name
-  -> ProcessConfig
+  -> CompleteProcessConfig
   -- ^ Process config
   -> IO ProcessHandle
-startProcess name ProcessConfig {..} = (\(_, _, _, x) -> x) <$>
-  createProcess_ name (proc name processConfigCmdLine)
-    { std_err = UseHandle processConfigStdErr
-    , std_out = UseHandle processConfigStdOut
-    , std_in  = UseHandle processConfigStdIn
-    , env     = Just processConfigEnvVars
+startProcess name CompleteProcessConfig {..} = (\(_, _, _, x) -> x) <$>
+  createProcess_ name (proc name completeProcessConfigCmdLine)
+    { std_err = UseHandle completeProcessConfigStdErr
+    , std_out = UseHandle completeProcessConfigStdOut
+    , std_in  = UseHandle completeProcessConfigStdIn
+    , env     = Just completeProcessConfigEnvVars
     }
 
+-- | Stop a 'ProcessHandle'. An alias for 'waitForProcess'
+stopProcess :: ProcessHandle -> IO ExitCode
+stopProcess = waitForProcess
+
 -- | Start a process and block until it finishes return the 'ExitCode'.
 executeProcess
   :: String
   -- ^ Process name
-  -> ProcessConfig
+  -> CompleteProcessConfig
   -- ^ Process config
   -> IO ExitCode
 executeProcess name = startProcess name >=> waitForProcess
 -------------------------------------------------------------------------------
 -- PostgresProcess Life cycle management
 -------------------------------------------------------------------------------
--- | 'PostgresPlan' is used be 'startPostgresProcess' to start the @postgres@
+-- | 'CompletePostgresPlan' is used be 'startPostgresProcess' to start the @postgres@
 --   and then attempt to connect to it.
-data PostgresPlan = PostgresPlan
-  { postgresPlanProcessConfig :: ProcessConfig
+data CompletePostgresPlan = CompletePostgresPlan
+  { completePostgresPlanProcessConfig :: CompleteProcessConfig
   -- ^ The process config for @postgres@
-  , postgresPlanClientOptions  :: Client.Options
+  , completePostgresPlanClientOptions  :: Client.Options
   -- ^ Connection options. Used to verify that @postgres@ is ready.
   }
 
-instance Pretty PostgresPlan where
-  pretty PostgresPlan {..}
-    =  text "postgresPlanProcessConfig:"
+instance Pretty CompletePostgresPlan where
+  pretty CompletePostgresPlan {..}
+    =  text "completePostgresPlanProcessConfig:"
     <> softline
-    <> indent 2 (pretty postgresPlanProcessConfig)
+    <> indent 2 (pretty completePostgresPlanProcessConfig)
     <> hardline
-    <> text "postgresPlanClientOptions:"
-    <+> prettyOptions postgresPlanClientOptions
+    <> text "completePostgresPlanClientOptions:"
+    <+> prettyOptions completePostgresPlanClientOptions
 
 prettyOptions :: Client.Options -> Doc
 prettyOptions = text . BSC.unpack . Client.toConnectionString
@@ -188,12 +205,12 @@ stopPostgresProcess PostgresProcess{..} = do
 -- | Start the @postgres@ process and block until a successful connection
 --   occurs. A separate thread we continously check to see if the @postgres@
 --   process has crashed.
-startPostgresProcess :: Logger -> PostgresPlan -> IO PostgresProcess
-startPostgresProcess logger PostgresPlan {..} = do
+startPostgresProcess :: Logger -> CompletePostgresPlan -> IO PostgresProcess
+startPostgresProcess logger CompletePostgresPlan {..} = do
   logger StartPostgres
 
-  let startAction = PostgresProcess postgresPlanClientOptions
-        <$> startProcess "postgres" postgresPlanProcessConfig
+  let startAction = PostgresProcess completePostgresPlanClientOptions
+        <$> startProcess "postgres" completePostgresPlanProcessConfig
 
   -- Start postgres and stop if an exception occurs
   bracketOnError startAction stopPostgresProcess $
@@ -206,54 +223,54 @@ startPostgresProcess logger PostgresPlan {..} = do
       logger WaitForDB
       -- We assume that 'template1' exist and make connection
       -- options to test if postgres is ready.
-      let options = postgresPlanClientOptions
+      let options = completePostgresPlanClientOptions
             { Client.dbname = pure "template1"
             }
 
       -- Block until a connection succeeds or postgres crashes
-      waitForDB options
+      waitForDB logger options
         `race_` forever (checkForCrash >> threadDelay 100000)
 
       -- Postgres is now ready so return
       return result
 -------------------------------------------------------------------------------
--- Plan
+-- CompletePlan
 -------------------------------------------------------------------------------
--- | 'Plan' is the low level configuration necessary for creating a database
+-- | 'CompletePlan' is the low level configuration necessary for creating a database
 --   starting @postgres@ and creating a database. There is no validation done
---   on the 'Plan'. It is recommend that one use the higher level functions
+--   on the 'CompletePlan'. It is recommend that one use the higher level functions
 --   such as 'Database.Postgres.Temp.start' which will generate plans that
---   are valid. 'Plan's are used internally but are exposed if the higher
+--   are valid. 'CompletePlan's are used internally but are exposed if the higher
 --   level plan generation is not sufficent.
-data Plan = Plan
-  { planLogger        :: Logger
-  , planInitDb        :: Maybe ProcessConfig
-  , planCreateDb      :: Maybe ProcessConfig
-  , planPostgres      :: PostgresPlan
-  , planConfig        :: String
-  , planDataDirectory :: FilePath
+data CompletePlan = CompletePlan
+  { completePlanLogger        :: Logger
+  , completePlanInitDb        :: Maybe CompleteProcessConfig
+  , completePlanCreateDb      :: Maybe CompleteProcessConfig
+  , completePlanPostgres      :: CompletePostgresPlan
+  , completePlanConfig        :: String
+  , completePlanDataDirectory :: FilePath
   }
 
-instance Pretty Plan where
-  pretty Plan {..}
-    =   text "planInitDb:"
+instance Pretty CompletePlan where
+  pretty CompletePlan {..}
+    =   text "completePlanInitDb:"
     <>  softline
-    <>  indent 2 (pretty planInitDb)
+    <>  indent 2 (pretty completePlanInitDb)
     <>  hardline
-    <>  text "planCreateDb:"
+    <>  text "completePlanCreateDb:"
     <>  softline
-    <>  indent 2 (pretty planCreateDb)
+    <>  indent 2 (pretty completePlanCreateDb)
     <>  hardline
-    <>  text "planPostgres:"
+    <>  text "completePlanPostgres:"
     <>  softline
-    <>  indent 2 (pretty planPostgres)
+    <>  indent 2 (pretty completePlanPostgres)
     <>  hardline
-    <>  text "planConfig:"
+    <>  text "completePlanConfig:"
     <>  softline
-    <>  indent 2 (pretty planConfig)
+    <>  indent 2 (pretty completePlanConfig)
     <>  hardline
-    <>  text "planDataDirectory:"
-    <+> pretty planDataDirectory
+    <>  text "completePlanDataDirectory:"
+    <+> pretty completePlanDataDirectory
 
 -- A simple helper to throw 'ExitCode's when they are 'ExitFailure'.
 throwIfNotSuccess :: Exception e => (ExitCode -> e) -> ExitCode -> IO ()
@@ -266,18 +283,19 @@ throwIfNotSuccess f = \case
 --   Additionally it writes a \"postgresql.conf\" and does not return until
 --   the @postgres@ process is ready. See 'startPostgresProcess' for more
 --   details.
-startPlan :: Plan -> IO PostgresProcess
-startPlan Plan {..} = do
-  for_ planInitDb  $ executeProcess "initdb" >=>
+startPlan :: CompletePlan -> IO PostgresProcess
+startPlan plan@CompletePlan {..} = do
+  completePlanLogger $ StartPlan $ show $ pretty plan
+  for_ completePlanInitDb  $ executeProcess "initdb" >=>
     throwIfNotSuccess InitDbFailed
 
   -- We must provide a config file before we can start postgres.
-  writeFile (planDataDirectory <> "/postgresql.conf") planConfig
+  writeFile (completePlanDataDirectory <> "/postgresql.conf") completePlanConfig
 
-  let startAction = startPostgresProcess planLogger planPostgres
+  let startAction = startPostgresProcess completePlanLogger completePlanPostgres
 
   bracketOnError startAction stopPostgresProcess $ \result -> do
-    for_ planCreateDb $  executeProcess "createdb" >=>
+    for_ completePlanCreateDb $  executeProcess "createdb" >=>
       throwIfNotSuccess CreateDbFailed
 
     pure result
