@@ -1,6 +1,6 @@
 import Test.Hspec
 import Database.Postgres.Temp.Internal.Core
-import Database.Postgres.Temp.Internal.Partial
+import Database.Postgres.Temp.Internal.Config
 import Database.Postgres.Temp.Internal
 import Control.Exception
 import System.IO.Error
@@ -29,10 +29,10 @@ main = hspec spec
 
 -- Cleanup
 
-fromCreateDb :: Maybe PartialProcessConfig -> Config
+fromCreateDb :: Maybe ProcessConfig -> Config
 fromCreateDb createDb = mempty
-  { configPlan = mempty
-      { partialPlanCreateDb = createDb
+  { plan = mempty
+      { createDbConfig = createDb
       }
   }
 
@@ -45,20 +45,20 @@ defaultConfigShouldMatchDefaultPlan :: SpecWith Runner
 defaultConfigShouldMatchDefaultPlan =
   it "default options should match default plan" $ withRunner $ \DB{..} -> do
     let Resources {..} = dbResources
-        Plan {..} = resourcesPlan
-        PostgresPlan {..} = planPostgres
-    Client.dbname postgresPlanClientOptions `shouldBe` pure "postgres"
-    let Temporary tmpDataDir = resourcesDataDir
+        CompletePlan {..} = resourcesPlan
+        CompletePostgresPlan {..} = completePlanPostgres
+    Client.dbname completePostgresPlanClientOptions `shouldBe` pure "postgres"
+    let CTemporary tmpDataDir = resourcesDataDir
     tmpDataDir `shouldStartWith` "/tmp/tmp-postgres-data"
-    let Just port = getLast $ Client.port postgresPlanClientOptions
+    let Just port = getLast $ Client.port completePostgresPlanClientOptions
     port `shouldSatisfy` (>32768)
-    let UnixSocket (Temporary unixSocket) = resourcesSocket
+    let CUnixSocket (CTemporary unixSocket) = resourcesSocket
     unixSocket `shouldStartWith` "/tmp/tmp-postgres-socket"
-    postgresPlanClientOptions `shouldBe`
+    completePostgresPlanClientOptions `shouldBe`
       (mempty
-        { Client.port = Client.port postgresPlanClientOptions
-        , Client.host = Client.host postgresPlanClientOptions
-        , Client.dbname = Client.dbname postgresPlanClientOptions
+        { Client.port = Client.port completePostgresPlanClientOptions
+        , Client.host = Client.host completePostgresPlanClientOptions
+        , Client.dbname = Client.dbname completePostgresPlanClientOptions
         }
       )
 
@@ -70,64 +70,69 @@ customConfigWork action = do
       expectedDuration = "100ms"
       extraConfig = "log_min_duration_statement='" <> expectedDuration <> "'"
 
-  it "returns the right client options for the plan" $ do
+  it "returns the right client options for the plan" $ withTempDirectory "/tmp" "tmp-postgres-spec" $ \tmpDir -> do
     let customPlan = mempty
-          { configPlan = mempty
-              { partialPlanPostgres = mempty
-                  { partialPostgresPlanClientConfig = mempty
+          { temporaryDirectory = pure tmpDir
+          , plan = mempty
+              { postgresPlan = mempty
+                  { connectionOptions = mempty
                       { Client.user     = pure expectedUser
                       , Client.password = pure expectedPassword
                       , Client.dbname   = pure expectedDbName
                       }
                   }
-              , partialPlanInitDb = pure standardProcessConfig
-                  { partialProcessConfigCmdLine = mempty
-                      { partialCommandLineArgsKeyBased =
+              , initDbConfig = pure standardProcessConfig
+                  { commandLine = mempty
+                      { keyBased =
                           Map.singleton "--username=" $ Just "user-name"
                       }
-                  , partialProcessConfigEnvVars = mempty
-                      { partialEnvVarsSpecific = Map.singleton "PGPASSWORD" "password"
-                      , partialEnvVarsInherit = pure True
+                  , environmentVariables = mempty
+                      { specific = Map.singleton "PGPASSWORD" "password"
+                      , inherit = pure True
                       }
                   }
-              , partialPlanCreateDb = pure standardProcessConfig
-                { partialProcessConfigCmdLine = mempty
-                  { partialCommandLineArgsKeyBased =
+              , createDbConfig = pure standardProcessConfig
+                { commandLine = mempty
+                  { keyBased =
                       Map.singleton "--username=" $ Just "user-name"
-                  , partialCommandLineArgsIndexBased =
+                  , indexBased =
                       Map.singleton 0 expectedDbName
                   }
-                , partialProcessConfigEnvVars =
+                , environmentVariables =
                     mempty
-                      { partialEnvVarsSpecific = Map.singleton "PGPASSWORD" "password"
-                      , partialEnvVarsInherit = pure True
+                      { specific = Map.singleton "PGPASSWORD" "password"
+                      , inherit = pure True
                       }
                 }
-              , partialPlanConfig = [extraConfig]
+              , postgresConfigFile = [extraConfig]
               }
           }
     -- hmm maybe I should provide lenses
     let combinedResources = defaultConfig <> customPlan
 
+    initialFiles <- listDirectory tmpDir
+
     action combinedResources $ \db@DB {..} -> do
       bracket (PG.connectPostgreSQL $ toConnectionString db) PG.close $ \conn -> do
         [PG.Only actualDuration] <- PG.query_ conn "SHOW log_min_duration_statement"
         actualDuration `shouldBe` expectedDuration
 
       let Resources {..} = dbResources
-          Plan {..} = resourcesPlan
-          actualOptions = postgresPlanClientOptions planPostgres
-          actualPostgresConfig = planConfig
+          CompletePlan {..} = resourcesPlan
+          actualOptions = completePostgresPlanClientOptions completePlanPostgres
+          actualPostgresConfig = completePlanConfig
       Client.user actualOptions `shouldBe` pure expectedUser
       Client.dbname actualOptions `shouldBe` pure expectedDbName
       Client.password actualOptions `shouldBe` pure expectedPassword
       lines actualPostgresConfig `shouldContain` defaultPostgresConfig <> [extraConfig]
 
+    listDirectory tmpDir `shouldReturn` initialFiles
+
 invalidConfigFailsQuickly :: (Config -> IO ()) -> Spec
 invalidConfigFailsQuickly action = it "quickly fails with an invalid option" $ do
   let customPlan = mempty
-        { configPlan = mempty
-            { partialPlanConfig =
+        { plan = mempty
+            { postgresConfigFile =
                 [ "log_directory = /this/does/not/exist"
                 , "logging_collector = true"
                 ]
@@ -175,11 +180,6 @@ withAnyPlan = do
 
     one `shouldBe` (1 :: Int)
 
-  it "cleans up temp files" $ \(Runner runner) -> do
-    initialFiles <- listDirectory "/tmp"
-    runner $ const $ pure ()
-    listDirectory "/tmp" `shouldReturn` initialFiles
-
 -- This assumes that the directory is initially empty
 withInitDbEmptyInitially :: SpecWith Runner
 withInitDbEmptyInitially = describe "with active initDb non-empty folder initially" $
@@ -203,11 +203,11 @@ createDbThrowsIfTheDbExists = describe "createdb" $
 spec :: Spec
 spec = do
   let defaultIpPlan = defaultConfig
-        { configSocket = PIpSocket $ Last Nothing
+        { socketClass = IpSocket $ Last Nothing
         }
 
       specificHostIpPlan = defaultConfig
-        { configSocket = PIpSocket $ pure "localhost"
+        { socketClass = IpSocket $ pure "localhost"
         }
 
   describe "start" $ do
@@ -217,8 +217,8 @@ spec = do
     let startAction plan = bracket (either throwIO pure =<< startConfig plan) stop pure
     throwsIfInitDbIsNotOnThePath $ startAction defaultConfig
     invalidConfigFailsQuickly $ void . startAction
-    customConfigWork $ \plan@Config{..} f ->
-      bracket (either throwIO pure =<< startConfig plan) stop f
+    customConfigWork $ \config@Config{..} f ->
+      bracket (either throwIO pure =<< startConfig config) stop f
   describe "with" $ do
     let startAction = either throwIO pure =<< with (const $ pure ())
     throwsIfInitDbIsNotOnThePath startAction
@@ -277,8 +277,8 @@ spec = do
 
     let invalidCreateDbPlan = defaultConfig <> fromCreateDb
           ( pure $ standardProcessConfig
-              { partialProcessConfigCmdLine = mempty
-                { partialCommandLineArgsIndexBased =
+              { commandLine = mempty
+                { indexBased =
                     Map.singleton 0 "template1"
                 }
               }
@@ -287,10 +287,10 @@ spec = do
       createDbThrowsIfTheDbExists
 
     let noCreateTemplate1 = mempty
-          { configPlan = mempty
-              { partialPlanCreateDb = Nothing
-              , partialPlanPostgres = mempty
-                  { partialPostgresPlanClientConfig = mempty
+          { plan = mempty
+              { createDbConfig = Nothing
+              , postgresPlan = mempty
+                  { connectionOptions = mempty
                     { Client.dbname = pure "template1"
                     }
                   }
@@ -320,7 +320,7 @@ spec = do
       it "fails on non-empty data directory" $ \dirPath -> do
         writeFile (dirPath <> "/PG_VERSION") "1 million"
         let nonEmptyFolderPlan = defaultConfig
-              { configDataDir = PPermanent dirPath
+              { dataDirectory = Permanent dirPath
               }
             startAction = bracket (either throwIO pure =<< startConfig nonEmptyFolderPlan) stop $ const $ pure ()
 
@@ -329,9 +329,9 @@ spec = do
       it "works if on non-empty if initdb is disabled" $ \dirPath -> do
         throwIfNotSuccess id =<< system ("initdb " <> dirPath)
         let nonEmptyFolderPlan = defaultConfig
-              { configDataDir = PPermanent dirPath
-              , configPlan = (configPlan defaultConfig)
-                  { partialPlanInitDb = Nothing
+              { dataDirectory = Permanent dirPath
+              , plan = (plan defaultConfig)
+                  { initDbConfig = Nothing
                   }
               }
         bracket (either throwIO pure =<< startConfig nonEmptyFolderPlan) stop $ \db -> do
@@ -342,8 +342,8 @@ spec = do
           one `shouldBe` (1 :: Int)
 
     let justBackupResources = mempty
-          { configPlan = mempty
-              { partialPlanConfig =
+          { plan = mempty
+              { postgresConfigFile =
                   [ "wal_level=replica"
                   , "archive_mode=on"
                   , "max_wal_senders=2"
